<?xml version = "1.0"?>

<xsl:stylesheet version = "1.0" exclude-result-prefixes = "h"
   xmlns:xsl   = "http://www.w3.org/1999/XSL/Transform"
   xmlns:msxsl = "urn:schemas-microsoft-com:xslt"
   xmlns:po    = "http://www.w3.org/1998/Math/MathML/PresentationObjects"
   xmlns:h     = "http://www.w3.org/1999/xhtml"
   xmlns:doc   = "http://www.w3.org/XSL/Annotation"
>
   <xsl:output method = "xml"/>

   <doc:section>Presentation Objects</doc:section>
   <doc:description>
      This is an intermediate form between MathML Presentation Markup and HTML+CSS rendering. There are
      several reasons for this:
      <doc:list>
         <doc:item>
            To simplify the rendering model by converting as many elements as possible into a span element
            with appropriate styling. This makes it easier to render as you are now working on a reduced
            element set.
         </doc:item><doc:item>
            To apply style inheritance: when using tables in the rendering implementation IE6 does not
            inherit styles from the outer element to a tables td element. I don't know if this is just
            IE6 or if it is part of the HTML4 specification or what.
         </doc:item>
      </doc:list>
   </doc:description>

   <doc:section>Font Scaling</doc:section>
   <doc:description>
      The following define the rules on how the fonts should be scaled relative to the containing elements
      sont size.
   </doc:description>

   <xsl:template match = "
      po:subsup/po:*[2] | po:subsup/po:*[3] | po:underover/po:*[2] | po:underover/po:*[3] |
      po:fraction[
         ancestor::po:fraction  or ancestor::po:radical    or ancestor::po:underover or
         ancestor::po:prescript or ancestor::po:postscript or
         ancestor::po:subsup    or ancestor::po:sub        or ancestor::po:sup
      ]/po:*[1] | po:fraction[
         ancestor::po:fraction  or ancestor::po:radical    or ancestor::po:underover or
         ancestor::po:prescript or ancestor::po:postscript or
         ancestor::po:subsup    or ancestor::po:sub        or ancestor::po:sup
      ]/po:*[2] |
      po:radical/po:*[2] | po:prescript/po:* | po:postscript/po:*
   " mode = "po-fontscale">
      0.5
   </xsl:template>

   <xsl:template match = "
      po:sub/po:*[2] | po:sup/po:*[2]
   " mode = "po-fontscale">
      0.75
   </xsl:template>

   <xsl:template match = "po:*" mode = "po-fontscale">
      1
   </xsl:template>

   <doc:section>Attribute Inheritance</doc:section>
   <doc:description>
      The algorithm for attribute inheritance is thus.
      <doc:list>
         <doc:item>
            Grab the attributes from the parent node (including inherited ones) into a variable. This is
            to allow the proper inheritance of the scaled font size.
         </doc:item><doc:item>
            Copy over the parents attributes, then add the attributes defined in the current node, excluding
            border, margin and padding attributes.
         </doc:item><doc:item>
            If we have not overwritten the font-size in this element, pull in the inherited value and scale it
            according to its markup.
         </doc:item>
      </doc:list>
   </doc:description>

   <xsl:template match = "po:*" mode = "inherit-attr">
      <xsl:variable name = "a">
         <xsl:element name = "attr"><xsl:choose>
            <xsl:when test = "parent::po:*">
               <xsl:apply-templates select = "parent::po:*" mode = "inherit-attr"/>
            </xsl:when><xsl:otherwise>
               <xsl:if test = "not(@font-family)">
                  <xsl:attribute name = "font-family">Times New Roman</xsl:attribute>
               </xsl:if>
            </xsl:otherwise>
         </xsl:choose></xsl:element>
      </xsl:variable>
      <!-- [review]: fix to support other XSLT engines other than MSXML -->
      <xsl:copy-of select = "msxsl:node-set($a)/attr/@*"/> 
      <xsl:copy-of select = "@*[
         not(local-name(.)='bevelled') and
         not(contains(local-name(.), 'border')) and
         not(contains(local-name(.), 'margin')) and
         not(contains(local-name(.), 'padding'))
      ]"/>

      <xsl:choose>
         <xsl:when test = "@font-size"/>
         <xsl:when test = "msxsl:node-set($a)/attr/@font-size">
            <!-- [review]: fix to support other XSLT engines other than MSXML -->
            <xsl:attribute name = "font-size"><xsl:call-template name = "scale-csslength">
               <xsl:with-param name = "size"><xsl:value-of select = "msxsl:node-set($a)/attr/@font-size"/></xsl:with-param>
               <xsl:with-param name = "scale"><xsl:apply-templates select = "." mode = "po-fontscale"/></xsl:with-param>
            </xsl:call-template></xsl:attribute>
         </xsl:when>
      </xsl:choose>
   </xsl:template>

   <doc:section>apply attribute inheritance</doc:section>

   <xsl:template match = "po:*" mode = "inherit">
      <xsl:copy><xsl:apply-templates select = "." mode = "inherit-attr"/>
         <xsl:apply-templates mode = "inherit"/>
      </xsl:copy>
   </xsl:template>

   <doc:section>special case: po:span element</doc:section>

   <xsl:template match = "po:span" mode = "inherit">
      <xsl:copy>
         <xsl:for-each select = "@*[
            contains(local-name(.), 'border') or
            contains(local-name(.), 'margin') or
            contains(local-name(.), 'padding')
         ]">
            <xsl:copy-of select = "."/>
         </xsl:for-each>
         <xsl:apply-templates select = "." mode = "inherit-attr"/>

         <xsl:apply-templates mode = "inherit"/>
      </xsl:copy>
   </xsl:template>

   <doc:section>special case: po:fraction element</doc:section>

   <xsl:template match = "po:fraction" mode = "inherit">
      <xsl:copy><xsl:copy-of select = "@bevelled"/>
         <xsl:apply-templates select = "." mode = "inherit-attr"/>
         
         <xsl:apply-templates mode = "inherit"/>
      </xsl:copy>
   </xsl:template>

   <doc:section>scale a CSS length attribute by a given amount</doc:section>

   <xsl:template name = "scale-csslength"> <xsl:param name = "size"/> <xsl:param name = "scale"/>
      <xsl:variable name = "ns"><xsl:value-of select = "normalize-space($size)"/></xsl:variable>

      <xsl:choose>
         <xsl:when test = "contains($size, '%')">
            <xsl:value-of select = "number(substring-before($size, '%')) * $scale"/>%
         </xsl:when><xsl:when test = "contains($size, 'em')">
            <xsl:value-of select = "number(substring-before($size, 'em')) * $scale"/>em
         </xsl:when><xsl:when test = "contains($size, 'ex')">
            <xsl:value-of select = "number(substring-before($size, 'ex')) * $scale"/>ex
         </xsl:when><xsl:when test = "contains($size, 'px')">
            <xsl:value-of select = "number(substring-before($size, 'px')) * $scale"/>px
         </xsl:when><xsl:when test = "contains($size, 'in')">
            <xsl:value-of select = "number(substring-before($size, 'in')) * $scale"/>in
         </xsl:when><xsl:when test = "contains($size, 'cm')">
            <xsl:value-of select = "number(substring-before($size, 'cm')) * $scale"/>cm
         </xsl:when><xsl:when test = "contains($size, 'mm')">
            <xsl:value-of select = "number(substring-before($size, 'mm')) * $scale"/>mm
         </xsl:when><xsl:when test = "contains($size, 'pt')">
            <xsl:value-of select = "number(substring-before($size, 'pt')) * $scale"/>pt
         </xsl:when><xsl:when test = "contains($size, 'pc')">
            <xsl:value-of select = "number(substring-before($size, 'pc')) * $scale"/>pc
         </xsl:when><xsl:otherwise>
            <xsl:value-of select = "number($size) * $scale"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <doc:section>construct a style element from the nodes attributes</doc:section>

   <xsl:template match = "*" mode = "po-getstyle" name = "po-getstyle">
      <xsl:attribute name = "style">
         <xsl:for-each select = "@*[not(local-name(.)='bevelled')]"><xsl:value-of select = "
            local-name(.)
         "/>: <xsl:value-of select = "
            normalize-space(.)
         "/>;<xsl:if test = "not(position()=last())"><xsl:text> </xsl:text></xsl:if></xsl:for-each>
      </xsl:attribute>
   </xsl:template>

   <doc:section>Driver</doc:section>
   <doc:description>
      This is the main driver for rendering a document. The first step is to inherit attributes, especially
      those that are style-related; the second step is to then render the document.
   </doc:description>

   <xsl:template match = "po:display" mode = "po-view">
      <xsl:variable name = "po">
         <xsl:apply-templates select = "." mode = "inherit"/>
      </xsl:variable>
      <!-- [review]: fix to support other XSLT engines other than MSXML -->
      <xsl:apply-templates select = "msxsl:node-set($po)" mode = "render"/>
   </xsl:template>

   <doc:section>renderer - display the presentation objects</doc:section>

   <doc:section>text elements</doc:section>

   <xsl:template match = "text()" mode = "render">
      <xsl:apply-templates select = "."/>
   </xsl:template>

   <doc:section>po:display elements</doc:section>

   <xsl:template match = "po:display" mode = "render">
      <span><xsl:call-template name = "po-getstyle"/>
         <xsl:apply-templates mode = "render"/>
      </span>
   </xsl:template>

   <doc:section>po:span elements</doc:section>

   <xsl:template match = "po:span" mode = "render">
      <span><xsl:call-template name = "po-getstyle"/>
         <xsl:apply-templates mode = "render"/>
      </span>
   </xsl:template>

   <doc:section>po:operator elements</doc:section>

   <xsl:template match = "po:operator" mode = "render">
      <span><xsl:call-template name = "po-getstyle"/>
         <xsl:apply-templates mode = "render"/>
      </span>
   </xsl:template>

   <doc:section>po:error elements</doc:section>

   <xsl:template match = "po:error" mode = "render">
      <span><xsl:call-template name = "po-getstyle"/>
         <span style = "color: red;">
            <xsl:apply-templates mode = "render"/>
         </span>
      </span>
   </xsl:template>

   <doc:section>po:fraction elements</doc:section>
   <h:p>[fixme]: line thickness</h:p>

   <h:p>does the fraction have a fraction as its ancestor?</h:p>

   <xsl:template match = "po:*" mode = "fraction">
      <xsl:apply-templates select = "ancestor::*[1]" mode = "fraction"/>
   </xsl:template>
   <xsl:template match = "po:fraction" mode = "fraction">true</xsl:template>
   <xsl:template match = "po:display"  mode = "fraction">false</xsl:template>

   <h:p>standard fraction</h:p>

   <xsl:template match = "po:fraction[@bevelled='false']" mode = "render">
      <table style = "text-align: center; display: inline; vertical-align: middle;" border = "0">
         <tr>
            <td valign = "bottom" align = "center" style = "border-width: 0em 0em .2ex 0em; border-style: solid;">
               <span><xsl:call-template name = "po-getstyle"/>
                  <xsl:apply-templates select = "*[1]" mode = "render"/>
               </span>
            </td>
         </tr><tr>
            <td valign = "top" align = "center"><xsl:call-template name = "po-getstyle"/>
               <xsl:apply-templates select = "*[2]" mode = "render"/>
            </td>
         </tr>
      </table>
   </xsl:template>

   <h:p>bevelled fraction</h:p>

   <xsl:template match = "po:fraction[@bevelled='true']" mode = "render">
      <xsl:variable name = "a"><xsl:apply-templates select = "ancestor::*[1]" mode = "fraction"/></xsl:variable>
      <span><xsl:call-template name = "po-getstyle"/>
         <xsl:choose>
            <xsl:when test = "$a = 'true'">
               <sup><xsl:apply-templates select = "*[1]" mode = "render"/></sup>
               <span>/</span>
               <sub><xsl:apply-templates select = "*[2]" mode = "render"/></sub>
            </xsl:when><xsl:otherwise>
               <sup><xsl:apply-templates select = "*[1]" mode = "render"/></sup>
               <span style = "vertical-align: middle; font-size: 2em;">/</span>
               <sub><xsl:apply-templates select = "*[2]" mode = "render"/></sub>
            </xsl:otherwise>
         </xsl:choose>
      </span>
   </xsl:template>

   <doc:section>po:sqrt elements</doc:section>

   <xsl:template match = "po:sqrt" mode = "render">
      <span>
         <span style = "vertical-align: top; font-family: 'StarMath';">&#x87;&#xFEFF;</span>
         <span style = "display: inline;
            border-style: solid; border-width: 1pt 0pt 0pt 0pt;
            padding-left: .2em; margin-left: 0em; margin-top: .2em; height: 1em;
         "><span><xsl:call-template name = "po-getstyle"/>
            <xsl:apply-templates mode = "render"/>
         </span></span>
      </span>
   </xsl:template>

   <doc:section>po:radical elements</doc:section>

   <xsl:template match = "po:radical" mode = "render">
      <span>
         <span style = "vertical-align: top;">
            <sup><xsl:apply-templates select = "*[2]"/></sup>
            <span style = "font-family: 'StarMath';">&#x87;&#xFEFF;</span>
         </span><span style = "display: inline;
            border-style: solid; border-width: 1pt 0pt 0pt 0pt;
            padding-left: .2em; margin-left: 0em; margin-top: .2em; height: 1em;
         "><span><xsl:call-template name = "po-getstyle"/>
            <xsl:apply-templates select = "*[1]" mode = "render"/>
         </span></span>
      </span>
   </xsl:template>

   <doc:section>po:subsup elements</doc:section>

   <xsl:template match = "po:subsup" mode = "render">
      <span><xsl:call-template name = "po-getstyle"/>
         <xsl:apply-templates select = "*[1]" mode = "render"/>

         <table cellspacing = "0" cellpadding = "0" style = "display: inline; vertical-align: middle;">
            <tr><td><xsl:apply-templates select = "*[3]" mode = "render"/></td></tr>
            <tr><td>&#xA0;</td></tr>
            <tr><td><xsl:apply-templates select = "*[2]" mode = "render"/></td></tr>
         </table>
      </span>
   </xsl:template>

   <doc:section>po:sub elements</doc:section>

   <xsl:template match = "po:sub" mode = "render">
      <xsl:apply-templates select = "*[1]" mode = "render"/>
      <sub><xsl:call-template name = "po-getstyle"/><xsl:apply-templates select = "*[2]" mode = "render"/></sub>
   </xsl:template>

   <doc:section>po:sup elements</doc:section>

   <xsl:template match = "po:sup" mode = "render">
      <xsl:apply-templates select = "*[1]" mode = "render"/>
      <sup><xsl:call-template name = "po-getstyle"/><xsl:apply-templates select = "*[2]" mode = "render"/></sup>
   </xsl:template>

   <doc:section>po:underover elements</doc:section>

   <xsl:template match = "po:underover" mode = "render">
      <table cellspacing = "0" cellpadding = "0" style = "display: inline; vertical-align: middle;">
         <tr>
            <td valign = "bottom" align = "center"><xsl:call-template name = "po-getstyle"/>
               <xsl:apply-templates select = "*[3]" mode = "render"/>
            </td>
         </tr><tr>
            <td valign = "middle" align = "center"><xsl:call-template name = "po-getstyle"/>
               <xsl:apply-templates select = "*[1]" mode = "render"/>
            </td>
         </tr><tr>
            <td valign = "top" align = "center"><xsl:call-template name = "po-getstyle"/>
               <xsl:apply-templates select = "*[2]" mode = "render"/>
            </td>
         </tr>
      </table>
   </xsl:template>

   <doc:section>po:multiscripts elements</doc:section>

   <xsl:template match = "po:multiscripts" mode = "render">
      <table style = "display: inline; vertical-align: middle;" cellspacing = "0" cellpadding = "0" border = "0">
         <tr>
            <xsl:for-each select = "po:prescript/*[position() mod 2 = 0]">
               <td><xsl:call-template name = "po-getstyle"/>
                  <xsl:apply-templates select = "." mode = "render"/>
               </td>
            </xsl:for-each>
            <td rowspan = "2">
               <xsl:apply-templates select = "*[1]" mode = "render"/>
            </td>
            <xsl:for-each select = "po:postscript/*[position() mod 2 = 0]">
               <td><xsl:call-template name = "po-getstyle"/>
                  <xsl:apply-templates select = "." mode = "render"/>
               </td>
            </xsl:for-each>
         </tr><tr>
            <xsl:for-each select = "po:prescript/*[position() mod 2 = 1]">
               <td><xsl:call-template name = "po-getstyle"/>
                  <xsl:apply-templates select = "." mode = "render"/>
               </td>
            </xsl:for-each>
            <xsl:for-each select = "po:postscript/*[position() mod 2 = 1]">
               <td><xsl:call-template name = "po-getstyle"/>
                  <xsl:apply-templates select = "." mode = "render"/>
               </td>
            </xsl:for-each>
         </tr>
      </table>
   </xsl:template>

   <doc:section>po:table elements</doc:section>

   <xsl:template match = "po:table" mode = "render">
      <table style = "display: inline; vertical-align: middle;" cellpadding = "0" cellspacing = "0" border = "0">
         <xsl:apply-templates mode = "render"/>
      </table>
   </xsl:template>

   <doc:section>po:tr elements</doc:section>

   <xsl:template match = "po:tr" mode = "render">
      <tr><xsl:apply-templates mode = "render"/></tr>
   </xsl:template>

   <doc:section>po:td elements</doc:section>

   <xsl:template match = "po:td" mode = "render">
      <td><xsl:apply-templates mode = "render"/></td>
   </xsl:template>
</xsl:stylesheet>
