<?xml version = "1.0" encoding = "iso-8859-1"?>

<xsl:stylesheet version = "1.0" exclude-result-prefixes = "h"
   xmlns:xsl = "http://www.w3.org/1999/XSL/Transform"
   xmlns:mml = "http://www.w3.org/1998/Math/MathML"
   xmlns:h   = "http://www.w3.org/1999/xhtml"
>
   <h:h2>Token Element Attributes</h:h2>

   <xsl:template match = "@*" mode = "po-style"/>

   <h:h3>style</h:h3>
   
   <h:p>
      Split the style element into attribute values so they can be inherited and manipulated by
      the po processor. They will be recombined later once the styles are properly inherited.
   </h:p>

   <xsl:template match = "@style" mode = "po-style"> <xsl:param name = "s"><xsl:value-of select = "normalize-space(.)"/></xsl:param>
      <xsl:variable name = "style"><xsl:choose>
         <xsl:when test = "contains($s, ';')"><xsl:value-of select = "normalize-space(substring-before(  $s, ';' ))"/></xsl:when>
         <xsl:otherwise><xsl:value-of select = "$s"/></xsl:otherwise>
      </xsl:choose></xsl:variable>
      <xsl:variable name = "rstyle"><xsl:choose>
         <xsl:when test = "contains($s, ';')"><xsl:value-of select = "normalize-space(substring-after(  $s, ';' ))"/></xsl:when>
         <xsl:otherwise/>
      </xsl:choose></xsl:variable>

      <xsl:variable name = "name" ><xsl:value-of select = "normalize-space(substring-before($style, ':'))"/></xsl:variable>
      <xsl:variable name = "value"><xsl:value-of select = "normalize-space(substring-after($style, ':'))"/></xsl:variable>

      <xsl:attribute name = "{$name}"><xsl:value-of select = "$value"/></xsl:attribute>
      <xsl:if test = "not($rstyle='')">
         <xsl:apply-templates select = "." mode = "po-style">
            <xsl:with-param name = "s"><xsl:value-of select = "$rstyle"/></xsl:with-param>
         </xsl:apply-templates>
      </xsl:if>
   </xsl:template>

   <h:h3>id | class</h:h3>

   <xsl:template match = "@id|@class" mode = "po-style">
      <xsl:copy-of select = "."/>
   </xsl:template>

   <h:h3>mathsize</h:h3>

   <xsl:template match = "@mathsize[
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-size"><xsl:value-of select = "."/></xsl:attribute>
   </xsl:template>

   <h:h3>mathcolor</h:h3>

   <xsl:template match = "@mathcolor[
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "color"><xsl:value-of select = "."/></xsl:attribute>
   </xsl:template>

   <h:h3>mathbackground</h:h3>

   <xsl:template match = "@mathbackground[
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "background-color"><xsl:value-of select = "."/></xsl:attribute>
   </xsl:template>

   <h:h3>mathvariant</h:h3>

   <xsl:template match = "@mathvariant[.='normal'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-weight">normal</xsl:attribute>
      <xsl:attribute name = "font-style" >normal</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='bold'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-weight">bold</xsl:attribute>
      <xsl:attribute name = "font-style" >upright</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='italic'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-style" >italic</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='bold-italic'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-weight">bold</xsl:attribute>
      <xsl:attribute name = "font-style" >italic</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='bold-fraktur'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-weight">bold</xsl:attribute>
      <xsl:attribute name = "font-family">euclid fraktur, old english text mt, amhurst sf, english towne-bold</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='script'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-family">euclid math one, brush script mt italic, magik, script</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='fraktur'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-family">euclid fraktur, old english text mt, amhurst sf, english towne-normal</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='sans-serif'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-family">sans-serif</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='bold-sans-serif'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-family">sans-serif</xsl:attribute>
      <xsl:attribute name = "font-weight">bold</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='sans-serif-italic'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-family">sans-serif</xsl:attribute>
      <xsl:attribute name = "font-style" >italic</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='sans-serif-bold-italic'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-family">sans-serif</xsl:attribute>
      <xsl:attribute name = "font-weight">bold</xsl:attribute>
      <xsl:attribute name = "font-style" >italic</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='monospace'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-family">monospace, courier</xsl:attribute>
   </xsl:template>

   <xsl:template match = "@mathvariant[.='double-struck'][
      parent::mml:mi    or parent::mml:mn     or parent::mml:mo or
      parent::mml:mtext or parent::mml:mspace or parent::mml:ms or parent::mml:mglyph
   ]" mode = "po-style">
      <xsl:attribute name = "font-family">euclid math two, Castellar, Algerian, Niagara Engraved, Casper Open SF</xsl:attribute>
   </xsl:template>
</xsl:stylesheet>
