<?xml version = "1.0"?>

<xsl:stylesheet version = "1.0"
   xmlns:xsl   = "http://www.w3.org/1999/XSL/Transform"
   xmlns:msxsl = "urn:schemas-microsoft-com:xslt"
   xmlns:dt    = "http://www.softwave.net/XML/DateTime"
>
   <!-- date == dt:day | dt:month | dt:year -->

   <!-- advance a date by the specified amount of days -->

   <xsl:template match = "*" mode = "date-advance"> <xsl:param name = "n">1</xsl:param>
      <xsl:copy><xsl:copy-of select = "@*"/>
         <xsl:if test = "@dt:day and @dt:month and @dt:year">
            <xsl:variable name = "mn"><xsl:call-template name = "month2number">
               <xsl:with-param name = "m"><xsl:value-of select = "@dt:month"/></xsl:with-param>
            </xsl:call-template></xsl:variable>

            <xsl:variable name = "dim"><xsl:call-template name = "numdays">
               <xsl:with-param name = "m"><xsl:value-of select = "$mn"/></xsl:with-param>
            </xsl:call-template></xsl:variable>

            <xsl:variable name = "dv"><xsl:value-of select = "number(@dt:day) + number($n)"/></xsl:variable>

            <xsl:variable name = "d">
               <xsl:if test = "$dv &lt;= $dim"><xsl:value-of select = "number($dv)"/></xsl:if>
               <xsl:if test = "$dv &gt;  $dim"><xsl:value-of select = "number($dv - $dim)"/></xsl:if>
            </xsl:variable><xsl:variable name = "m">
               <xsl:if test = "$dv &lt;= $dim"><xsl:value-of select = "number($mn)"/></xsl:if>
               <xsl:if test = "$dv &gt;  $dim">
                  <xsl:if test = "$mn &lt; 11"><xsl:value-of select = "number($mn + 1)"/></xsl:if>
                  <xsl:if test = "$mn = 11"   >0</xsl:if>
               </xsl:if>
            </xsl:variable><xsl:variable name = "y">
               <xsl:if test = "$dv &lt;= $dim"><xsl:value-of select = "number(@dt:year)"/></xsl:if>
               <xsl:if test = "$dv &gt;  $dim">
                  <xsl:if test = "$mn &lt; 11"><xsl:value-of select = "number(@dt:year)"/></xsl:if>
                  <xsl:if test = "$mn = 11"   ><xsl:value-of select = "number(@dt:year + 1)"/></xsl:if>
               </xsl:if>
            </xsl:variable>

            <xsl:attribute name = "dt:day"  ><xsl:value-of select = "$d" /></xsl:attribute>
            <xsl:attribute name = "dt:month">
               <xsl:call-template name = "month2string">
                  <xsl:with-param name = "m"><xsl:value-of select = "$m"/></xsl:with-param>
               </xsl:call-template>
            </xsl:attribute>
            <xsl:attribute name = "dt:year"><xsl:value-of select = "$y" /></xsl:attribute>
         </xsl:if>
      </xsl:copy>
   </xsl:template>

   <!-- calculate associated information relating to the day/month/year format -->

   <xsl:template match = "*" mode = "date-extendedinfo">
      <xsl:copy><xsl:copy-of select = "@*"/>
         <xsl:if test = "@dt:day and @dt:month and @dt:year">
            <xsl:variable name = "mn"><xsl:call-template name = "month2number">
               <xsl:with-param name = "m"><xsl:value-of select = "@dt:month"/></xsl:with-param>
            </xsl:call-template></xsl:variable>

            <xsl:variable name = "w"><xsl:call-template name = "dayofweek">
               <xsl:with-param name = "d"><xsl:value-of select = "@dt:day"  /></xsl:with-param>
               <xsl:with-param name = "m"><xsl:value-of select = "$mn"      /></xsl:with-param>
               <xsl:with-param name = "y"><xsl:value-of select = "@dt:year" /></xsl:with-param>
            </xsl:call-template></xsl:variable>

            <xsl:attribute name = "dt:monthno"><xsl:value-of select = "$mn" /></xsl:attribute>
            <xsl:attribute name = "dt:dow"    ><xsl:value-of select = "$w"  /></xsl:attribute>

            <xsl:attribute name = "dt:dowstring"><xsl:call-template name = "dayofweek2string">
               <xsl:with-param name = "dow"><xsl:value-of select = "$w"/></xsl:with-param>
            </xsl:call-template></xsl:attribute>
         </xsl:if>
      </xsl:copy>
   </xsl:template>

   <!-- produce a string representation of the date -->

   <xsl:template match = "*" mode = "date-tostring">

      <xsl:if test = "@dt:day and @dt:month and @dt:year">
         <xsl:if test = "@dt:dowstring"><xsl:value-of select = "
            @dt:dowstring
         "/>&#xA0;</xsl:if><xsl:value-of select = "
            @dt:day
         "/>&#xA0;<xsl:value-of select = "
            @dt:month
         "/>&#xA0;<xsl:value-of select = "
            @dt:year
         "/>
      </xsl:if>
   </xsl:template>

   <!-- calculate GMT or BST -->

   <xsl:template match = "*" mode = "date-gmtbst">
      <xsl:choose>
         <xsl:when test = "@dt:month = 'April'"    >bst</xsl:when>
         <xsl:when test = "@dt:month = 'May'"      >bst</xsl:when>
         <xsl:when test = "@dt:month = 'June'"     >bst</xsl:when>
         <xsl:when test = "@dt:month = 'July'"     >bst</xsl:when>
         <xsl:when test = "@dt:month = 'August'"   >bst</xsl:when>
         <xsl:when test = "@dt:month = 'September'">bst</xsl:when>
         <xsl:when test = "@dt:month = 'October' and (@dt:day &lt; 27)">bst</xsl:when>

         <!-- the day of the changeover: -->

         <xsl:when test = "@dt:month = 'October' and (@dt:day = '27')">bst</xsl:when>
         <xsl:when test = "@dt:month = 'March'   and (@dt:day = '31')">gmt</xsl:when>
         <xsl:otherwise>gmt</xsl:otherwise> <!-- default to GMT -->
      </xsl:choose>
   </xsl:template>

   <!-- date helper functions: conversions -->

   <xsl:template name = "month2number"> <xsl:param name = "m"/>
      <xsl:choose>
         <xsl:when test = "$m = 'January'"  >0</xsl:when>
         <xsl:when test = "$m = 'February'" >1</xsl:when>
         <xsl:when test = "$m = 'March'"    >2</xsl:when>
         <xsl:when test = "$m = 'April'"    >3</xsl:when>
         <xsl:when test = "$m = 'May'"      >4</xsl:when>
         <xsl:when test = "$m = 'June'"     >5</xsl:when>
         <xsl:when test = "$m = 'July'"     >6</xsl:when>
         <xsl:when test = "$m = 'August'"   >7</xsl:when>
         <xsl:when test = "$m = 'September'">8</xsl:when>
         <xsl:when test = "$m = 'October'"  >9</xsl:when>
         <xsl:when test = "$m = 'November'" >10</xsl:when>
         <xsl:when test = "$m = 'December'" >11</xsl:when>
      </xsl:choose>
   </xsl:template>

   <xsl:template name = "month2string"> <xsl:param name = "m"/>
      <xsl:choose>
         <xsl:when test = "$m = 0">January</xsl:when>
         <xsl:when test = "$m = 1">February</xsl:when>
         <xsl:when test = "$m = 2">March</xsl:when>
         <xsl:when test = "$m = 3">April</xsl:when>
         <xsl:when test = "$m = 4">May</xsl:when>
         <xsl:when test = "$m = 5">June</xsl:when>
         <xsl:when test = "$m = 6">July</xsl:when>
         <xsl:when test = "$m = 7">August</xsl:when>
         <xsl:when test = "$m = 8">September</xsl:when>
         <xsl:when test = "$m = 9">October</xsl:when>
         <xsl:when test = "$m = 10">November</xsl:when>
         <xsl:when test = "$m = 11">December</xsl:when>
      </xsl:choose>
   </xsl:template>

   <xsl:template name = "dayofweek2string"> <xsl:param name = "dow"/>
      <xsl:choose>
         <xsl:when test = "$dow = 0">Sunday</xsl:when>
         <xsl:when test = "$dow = 1">Monday</xsl:when>
         <xsl:when test = "$dow = 2">Tuesday</xsl:when>
         <xsl:when test = "$dow = 3">Wednesday</xsl:when>
         <xsl:when test = "$dow = 4">Thursday</xsl:when>
         <xsl:when test = "$dow = 5">Friday</xsl:when>
         <xsl:when test = "$dow = 6">Saturday</xsl:when>
      </xsl:choose>
   </xsl:template>

   <!-- date helper functions: number of days in a given month -->

   <xsl:template name = "numdays"> <xsl:param name = "m"/> <xsl:param name = "y"/>
      <xsl:choose>
         <xsl:when test = "$m = 1">
            <xsl:choose>
               <xsl:when test = "($y mod 400) = 0">29</xsl:when>
               <xsl:when test = "($y mod 100) = 0">28</xsl:when>
               <xsl:when test = "($y mod 4)   = 0">29</xsl:when>
               <xsl:otherwise>28</xsl:otherwise>
            </xsl:choose>
         </xsl:when>
         <xsl:when test = "$m = 3" >30</xsl:when>
         <xsl:when test = "$m = 5" >30</xsl:when>
         <xsl:when test = "$m = 8" >30</xsl:when>
         <xsl:when test = "$m = 10">30</xsl:when>
         <xsl:otherwise>31</xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <!-- date helper functions: calculate the day of week -->

   <xsl:template name = "dayofweek"> <xsl:param name = "d"/> <xsl:param name = "m"/> <xsl:param name = "y"/>
      <xsl:variable name = "a"><xsl:value-of select = "floor((14 - $m) div 12)"/></xsl:variable>
      <xsl:variable name = "y2"><xsl:value-of select = "$y - $a"/></xsl:variable>
      <xsl:variable name = "m2"><xsl:value-of select = "$m + (12 * $a) - 2"/></xsl:variable>

      <xsl:value-of select = "
         ( $d + $y2 + floor($y2 div 4) - floor($y2 div 100) + floor($y2 div 400) + floor(( 31 * $m2) div 12))
         mod
         7
      "/>
   </xsl:template>
</xsl:stylesheet>
