<?xml version = "1.0"?>

<xsl:stylesheet version = "1.0"
   xmlns:xsl   = "http://www.w3.org/1999/XSL/Transform"
   xmlns:msxsl = "urn:schemas-microsoft-com:xslt"

   xmlns:mml   = "http://www.w3.org/1998/Math/MathML"
   xmlns:svg   = "http://www.w3.org/2000/svg"

   xmlns:vc    = "http://www.softwave.net/XML/Version"
   xmlns:s     = "http://www.softwave.net/XML/ScriptML"
   xmlns:d     = "http://www.softwave.net/XML/Document"
>
   <!--
      display and format an XML document so the structure is visible
         ==> general formatting rules for a generic XML document
         ==> xmlns support for w3c standatds  - mml; svg
         ==> xmlns support for custom layouts - s; d
         ==> special rules for layout of m:apply MathML element
         ==> special rules for my custom XML document structures - s; d
   -->

   <xsl:include href = "../w3c/unicode/text.xsl"/>

   <xsl:template match = "*" mode = "display-xml">
      <html><head>
         <link type = "text/css" rel = "stylesheet" href = "/css/default.css"/>
         <title><xsl:value-of select = "@title"/></title>
      </head><body scroll = "auto">
         <xsl:variable name = "dxml">
            <xsl:apply-templates select = "." mode = "xmltransform"/>
         </xsl:variable>

         <xsl:apply-templates select = "msxsl:node-set($dxml)" mode = "write"/>
      </body></html>
   </xsl:template>

   <!-- transform -->

   <xsl:template match = "comment()" mode = "xmltransform">
      <xsl:element name = "block"><xsl:attribute name = "render">div</xsl:attribute>
         <xsl:element name = "symbol">&lt;!--</xsl:element>
         <xsl:element name = "block"><xsl:attribute name = "render">indent</xsl:attribute>
            <xsl:element name = "comment"><xsl:value-of select = "."/></xsl:element>
         </xsl:element>
         <xsl:element name = "symbol">--&gt;</xsl:element>
      </xsl:element>
   </xsl:template>

   <xsl:template match = "text()" mode = "xmltransform">
      <xsl:choose>
         <xsl:when test = "
            (
               parent::s:dialog or parent::s:description or
               parent::d:paragraph or parent::d:li or parent::d:item or parent::d:note or
               parent::vc:comment
            )
            and count(parent::*/*) = 0
         ">
            <xsl:element name = "block"><xsl:attribute name = "render">indent</xsl:attribute>
               <xsl:element name = "text"><xsl:value-of select = "."/></xsl:element>
            </xsl:element>
         </xsl:when><xsl:otherwise>
            <xsl:element name = "text"><xsl:value-of select = "."/></xsl:element>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <xsl:template match = "@*" mode = "xmltransform">
      <xsl:element name = "block"><xsl:attribute name = "render">span</xsl:attribute>
         <xsl:element name = "attribute"><xsl:value-of select = "name()"/></xsl:element>
         <xsl:element name = "symbol"> = &quot;</xsl:element>
         <xsl:element name = "string"><xsl:value-of select = "."/></xsl:element>
         <xsl:element name = "symbol">&quot;</xsl:element>
      </xsl:element>
   </xsl:template>

   <xsl:template match = "*" mode = "xmltransform">
      <xsl:element name = "block">
         <xsl:choose>
            <xsl:when test = "self::s:dialog">
               <xsl:attribute name = "render">div</xsl:attribute>
            </xsl:when><xsl:when test = "
               (count(*|text()) > 1 and not(preceding-sibling::text())) or
               (count(*|text()) = 0 and     preceding-sibling::text())  or
               ((
                  self::d:i  or self::d:item or self::d:sup or self::d:paragraph or
                  self::d:b  or self::d:link or self::d:sub or self::d:section   or
                  self::d:u  or self::d:li   or
                  self::s:u  or self::s:part or self::s:chapter or self::s:description
               ) and count(*|text()) > 0) or
               (parent::mml:apply and count(*|text()) = 0 and position() = 1)
            ">
               <xsl:attribute name = "render">span</xsl:attribute>
            </xsl:when><xsl:otherwise>
               <xsl:attribute name = "render">div</xsl:attribute>
            </xsl:otherwise>
         </xsl:choose>

         <xsl:element name = "block"><xsl:attribute name = "render">span</xsl:attribute>
            <xsl:element name = "symbol">&lt;</xsl:element>
            <xsl:element name = "element"><xsl:value-of select = "name()"/></xsl:element>
            <xsl:choose>
               <xsl:when test = "self::mml:math">
                  <xsl:element name = "block"><xsl:attribute name = "render">span</xsl:attribute>
                     <xsl:element name = "attribute">xmlns:m</xsl:element>
                     <xsl:element name = "symbol"   > = &quot;</xsl:element>
                     <xsl:element name = "nsstring" >http://www.w3.org/1998/Math/MathML</xsl:element>
                     <xsl:element name = "symbol"   >&quot;</xsl:element>
                  </xsl:element>
               </xsl:when><xsl:when test = "self::svg:svg">
                  <xsl:element name = "block"><xsl:attribute name = "render">span</xsl:attribute>
                     <xsl:element name = "attribute">xmlns:svg</xsl:element>
                     <xsl:element name = "symbol"   > = &quot;</xsl:element>
                     <xsl:element name = "nsstring" >http://www.w3.org/2000/svg</xsl:element>
                     <xsl:element name = "symbol"   >&quot;</xsl:element>
                  </xsl:element>
               </xsl:when><xsl:when test = "self::s:script">
                  <xsl:element name = "block"><xsl:attribute name = "render">span</xsl:attribute>
                     <xsl:element name = "attribute">xmlns:s</xsl:element>
                     <xsl:element name = "symbol"   > = &quot;</xsl:element>
                     <xsl:element name = "nsstring" >http://www.softwave.net/XML/ScriptML</xsl:element>
                     <xsl:element name = "symbol"   >&quot;</xsl:element>
                  </xsl:element>
               </xsl:when><xsl:when test = "self::d:document">
                  <xsl:element name = "block"><xsl:attribute name = "render">span</xsl:attribute>
                     <xsl:element name = "attribute">xmlns:d</xsl:element>
                     <xsl:element name = "symbol"   > = &quot;</xsl:element>
                     <xsl:element name = "nsstring" >http://www.softwave.net/XML/Document</xsl:element>
                     <xsl:element name = "symbol"   >&quot;</xsl:element>
                  </xsl:element>
               </xsl:when>
            </xsl:choose>
            <xsl:apply-templates select = "@*" mode = "xmltransform"/>
            <xsl:choose>
               <xsl:when test = "count(*|text()) = 0">
                  <xsl:element name = "symbol">/&gt;</xsl:element>
               </xsl:when><xsl:otherwise>
                  <xsl:element name = "symbol">&gt;</xsl:element>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:element>
         <xsl:if test = "count(*|text()) > 0">
            <xsl:if test = "self::mml:apply and count(*[1]/*|*[1]/text()) = 0">
               <xsl:apply-templates select = "*[1]" mode = "xmltransform"/>
            </xsl:if>
            <xsl:element name = "block"><xsl:attribute name = "render">span</xsl:attribute>
               <xsl:choose>
                  <xsl:when test = "count(*) = 0">
                     <xsl:attribute name = "render">span</xsl:attribute>
                  </xsl:when><xsl:otherwise>
                     <xsl:attribute name = "render">indent</xsl:attribute>
                  </xsl:otherwise>
               </xsl:choose>

               <xsl:choose>
                  <xsl:when test = "self::mml:apply and count(*[1]/*|*[1]/text()) = 0">
                     <xsl:apply-templates select = "*[position()>1]" mode = "xmltransform"/>
                  </xsl:when><xsl:otherwise>
                     <xsl:apply-templates mode = "xmltransform"/>
                  </xsl:otherwise>
               </xsl:choose>
            </xsl:element>
            <xsl:element name = "block"><xsl:attribute name = "render">span</xsl:attribute>
               <xsl:element name = "symbol">&lt;/</xsl:element>
               <xsl:element name = "element"><xsl:value-of select = "name()"/></xsl:element>
               <xsl:element name = "symbol">&gt;</xsl:element>
            </xsl:element>
         </xsl:if>
      </xsl:element>
   </xsl:template>

   <!-- renderer: layout formatting -->

   <xsl:template match = "block[@render='span']" mode = "write">
      <span><xsl:apply-templates mode = "write"/></span>
   </xsl:template>

   <xsl:template match = "block[@render='div']" mode = "write">
      <div><xsl:apply-templates mode = "write"/></div>
   </xsl:template>

   <xsl:template match = "block[@render='indent']" mode = "write">
      <div style = "margin-left: 1pc;"><xsl:apply-templates mode = "write"/></div>
   </xsl:template>

   <!-- renderer: XML elements -->

   <xsl:template match = "symbol" mode = "write">
      <span style = "color: orange;"><xsl:apply-templates mode = "write"/></span>
   </xsl:template>

   <xsl:template match = "element" mode = "write">
      <span style = "color: red;"><xsl:apply-templates mode = "write"/></span>
   </xsl:template>

   <xsl:template match = "attribute" mode = "write">
      <span style = "color: lightskyblue;">&#xA0;<xsl:apply-templates mode = "write"/></span>
   </xsl:template>

   <xsl:template match = "string" mode = "write">
      <span style = "color: yellow;"><xsl:apply-templates mode = "write"/></span>
   </xsl:template>

   <xsl:template match = "nsstring" mode = "write">
      <span style = "color: magenta;"><xsl:apply-templates mode = "write"/></span>
   </xsl:template>

   <xsl:template match = "text|comment" mode = "write">
      <span style = "color: limegreen;"><xsl:apply-templates/></span>
   </xsl:template>
</xsl:stylesheet>
